# -*- coding: utf-8 -*-
import requests
from urllib.parse import parse_qsl, urlencode, urlparse, urlunparse

from odoo.addons.saas_worker.auth_util import SaasAuth
from odoo.addons.saas_worker.const import WEBSITE
from odoo import models
from odoo.tools import ormcache


class ResPartner(models.Model):
    _inherit = 'res.partner'

    # Used from action_reset_password and _compute_signup_url
    # action_reset_password will render a template containing the signup_url,
    # so both calls will be made very close in time.
    # Caching the results spares a second useless RPC call.
    @ormcache('self.user_ids[:1].login')
    def _user_exists_odoocom(self):
        self.ensure_one()
        if not self.user_ids:
            return False

        try:
            resp = requests.post(f'{WEBSITE}/openerp_enterprise/trial-exists',
                                 json={'params': {'login': self.user_ids[0].login}},
                                 auth=SaasAuth(),
                                 timeout=2)
            resp.raise_for_status()
            return bool(resp.json().get('result'))
        except Exception:
            #  ^ could be HTTPError, requests.exceptions.ConnectTimeout or even json.JSONDecodeError
            return False

    def _compute_signup_url(self):
        super()._compute_signup_url()

        oauth_provider_id_field = self.env['res.users']._fields.get('oauth_provider_id')
        provider_openerp = self.env.ref('auth_oauth.provider_openerp', raise_if_not_found=False)
        provider_openerp_ids = [False, provider_openerp.id] if provider_openerp else [False]

        for partner in self:
            is_internal = not partner.partner_share
            no_external_oauth = (partner.user_ids.oauth_provider_id.id in provider_openerp_ids
                                 if oauth_provider_id_field else True)
            # retarget the signup_url to odoo.com only for internal users who have a signup_token
            if is_internal and no_external_oauth and partner.signup_valid and partner._user_exists_odoocom():
                signup_url = urlparse(partner.signup_url)
                website_url = urlparse(WEBSITE)
                query = dict(parse_qsl(signup_url.query))
                query.setdefault('signup_email', partner.email)
                new_parts = dict(scheme=website_url.scheme,
                                 netloc=website_url.netloc,
                                 path='/openerp_enterprise/confirm_user',
                                 query=urlencode(query))
                partner.signup_url = urlunparse(signup_url._replace(**new_parts))
